var websocketServer = require("websocket").server;
var http = require("http");
var static = require('node-static');

var config = {
    serverPort: 12345,     // port for HTTP and websockets
    packetLength: 1000,    // length of a packet in bytes
    totalPackets: 10000,   // total number of packets to send
    packetsPerTimeout: 10, // number of packets to send at a time
    packetTimeout: 1       // ms between packet send events
}

var stats = {
    bytesSent: 0,
    packetsSent: 0
};

var wsConnection = null;
var testValue = 0;

// Host html and JS files for repro
var fileHost = new static.Server("./repro");

// Create an HTTP server for file hosting and websockets
var httpServer = http.createServer(function(request, response) {
    request.addListener('end', function() {
        fileHost.serve(request, response);
    }).resume();
}).listen(config.serverPort, function() {
    console.log("Awaiting connection on http://localhost:" + config.serverPort);
});

// Setup websockets server
var ws = new websocketServer({
    httpServer: httpServer,
    autoAcceptConnections: true
});

// Send the next sequence of packets to the websockets client
var SendNextPacket = function() {
    if (wsConnection) {
        if (stats.packetsSent < config.totalPackets) {
            // Send several packets containing sequential data
            for (var packetNum = 0; packetNum < config.packetsPerTimeout; packetNum++) {
                var data = new Buffer(config.packetLength);
                for (var i = 0; i < data.length; i++) {
                    data[i] = testValue;
                    testValue = (testValue + 1) & 0xFF;
                }
                wsConnection.sendBytes(data);
                stats.bytesSent += data.length;
                stats.packetsSent++;
            }

            // Await the next packet send timeout
            setTimeout(SendNextPacket, config.packetTimeout);
        } else {
            console.log("Sent " + stats.packetsSent + " packets for a total of " + stats.bytesSent + " bytes");

            // Close the websockets connection after sending all the packets
            setTimeout(function() { 
                if (wsConnection) {
                    wsConnection.close();
                    wsConnection = null;
                }
            }, 1000);
        }
    }
}

// Setup websockets connection
ws.on("connect", function(connection) {
    console.log("Connected");
    connection.binaryType = "arraybuffer";
    connection.on("message", function(message) {
        console.log("Message received: type " + message.type);
    });
    connection.on("close", function(reason, description) {
        console.log("Connection closed. Reason: " + reason + " description: " + description);
    });
    wsConnection = connection;
    stats.bytesSent = 0;
    stats.packetsSent = 0;
    testValue = 0;
    setTimeout(SendNextPacket, 1000);
});